import { world, system, EntityEquippableComponent, EquipmentSlot, ItemStack } from "@minecraft/server";
import { addDamage, removeEffect, addEffect, getScore, addScore, setScore } from './functions.js';


export function ambar_totem(eHurt) {
	let random_totem = Math.floor(Math.random() * 100)+1;
	let totem = getScore(eHurt, "ambar_totem")
	if(random_totem > 25 && random_totem < 66 && totem < 20){
		addEffect(eHurt, "health_boost", 10000000, totem, false)
		addEffect(eHurt, "regeneration", 40, totem+2, false)
		eHurt.runCommandAsync(`playsound random.totem_use @s`)
		eHurt.sendMessage(`§aThe totem was activated!`);
		setScore(eHurt, "ambar_totem", totem+1)
	}
	if(random_totem > 65 && totem < 20){
		eHurt.sendMessage(`§cThe totem failed!`);
	}
	if(random_totem < 26 && totem < 20){
		eHurt.runCommandAsync(`clear @s new:ambar_totem 0 1`);
		eHurt.runCommandAsync(`playsound random.break @s`)
		eHurt.sendMessage(`§dThe totem was broken!`);
	}
	if(totem >= 20){
		eHurt.sendMessage(`§bYou've reached your heart limit!!`);
	}
}

export function custom_trident(p, item){
	let damage = item.getComponent("durability").damage;
	let maxDurability = item.getComponent("durability").maxDurability;
	let trident_item = item.typeId;
	system.runTimeout(()=>{
		let equip = p.getComponent(EntityEquippableComponent.componentId);
		let hand = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(hand){
			if(hand.typeId == trident_item){
				p.removeTag("use_trident")
				let tridentItem = item.typeId.replace("rident", "")
				let trident = p.dimension.spawnEntity(`${tridentItem}`, p.getHeadLocation());
				let projectile = trident.getComponent('projectile');
				let itemTag = {
					'itemTag': {
						'id': `${item.typeId}`,
						'damage': `${damage+2}`
					}
				};
				trident.addTag(JSON.stringify(itemTag))
				projectile.owner = p;
				projectile.shoot({x: p.getViewDirection().x*4, y: p.getViewDirection().y*4, z: p.getViewDirection().z*4});
				p.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 air`)
				p.runCommandAsync("playsound item.trident.throw @s")
			}
			if(hand.typeId != trident_item){
				if(p.hasTag("use_trident")){
					p.removeTag("use_trident")
				}
			}
		}
		if(hand == undefined){
			if(p.hasTag("use_trident")){
				p.removeTag("use_trident")
			}
		}
	}, 1)
}

export function Durability(tool, source) {
	try{
		system.run(()=>{
			const component = source.getComponent(EntityEquippableComponent.componentId);
			const durability = tool.getComponent("durability");
			const enchantable = tool.getComponent("enchantable");
			if(tool && durability !== undefined){
				let unbreakingLevel = 0;
				const damageAmount = 2;
				if (enchantable) {
					for (const enchant of enchantable.getEnchantments()) {
						if (enchant.type.id === "unbreaking") {
							unbreakingLevel = enchant.level;
							break;
						}
					}
				}if(durability.damage+2 < durability.maxDurability){
					if(unbreakingLevel > 0) {
						const chance = 1 / (unbreakingLevel + 1);
						if (Math.random() < chance) {
							durability.damage += damageAmount;
						}
					} if(unbreakingLevel == 0) {
						durability.damage += damageAmount;
					}
					component.setEquipment(EquipmentSlot.Mainhand, tool)
				}else if(durability.damage+2 >= durability.maxDurability){
					const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Mainhand, air)
					source.playSound("random.break")
				}
			}
		})
	}catch(e){}
}

export function custom_bow(p, item){
	let bow_item = item.typeId;
	let inv = p.getComponent("inventory").container
	system.runTimeout(()=>{
		let equip = p.getComponent(EntityEquippableComponent.componentId);
		let hand = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(hand){
			if(hand.typeId == bow_item){
				let arrowItem = item.typeId.replace("bow", "arrow")
				for(let slot = 0; slot < inv.size; slot++){
					let itemInv = inv.getItem(slot);
					if(itemInv){
						if(itemInv.typeId == `${arrowItem}`){
							p.removeTag("use_bow")
						}
						if(itemInv.typeId != `${arrowItem}`){
							p.removeTag("use_bow")
						}
					}
					if(!itemInv){
						p.removeTag("use_bow")
					}
				}
			}
			if(hand.typeId != bow_item){
				if(p.hasTag("use_bow")){
					p.removeTag("use_bow")
				}
			}
		}
		if(hand == undefined){
			if(p.hasTag("use_bow")){
				p.removeTag("use_bow")
			}
		}
	}, 1)
}

export function custom_crossbow(p, itemStack) {
	if(itemStack.typeId.includes("_crossbow_arrow") && !p.hasTag("cooldown")){
		let item = itemStack.typeId.replace("_arrow", "_off")
		let arrowItem = itemStack.typeId.replace("_crossbow", "")
		let arrow = p.dimension.spawnEntity(`${arrowItem}`, p.getHeadLocation());
		let projectile = arrow.getComponent('projectile');
		projectile.owner = p;
		projectile.shoot({x: p.getViewDirection().x*3, y: p.getViewDirection().y*3, z: p.getViewDirection().z*3});
		p.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 ${item} 1 ${damage+2}`)
		p.runCommandAsync("playsound crossbow.shoot @s")
		p.addTag("cooldown")
		if(damage >= maxDurability){
			p.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 air`)
			p.runCommandAsync("playsound random.item_break @s ~~~")
		}
		system.runTimeout(()=>{
			p.removeTag("cooldown")
		}, 20)
	}
	if(itemStack.typeId.includes("_crossbow_off") && !p.hasTag("cooldown")){
		let inv = p.getComponent("inventory").container;
		let awItem = itemStack.typeId.replace("_crossbow_off", "_arrow")
		let item = itemStack.typeId.replace("_off", "_arrow")
		for(let slot = 0; slot < inv.size; slot++){
			let itemInv = inv.getItem(slot);
			if(itemInv){
				if(itemInv.typeId == `${awItem}`){
					p.runCommandAsync(`clear @s ${awItem} 0 1`)
					p.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 ${item} 1 ${damage}`)
					p.runCommandAsync("playsound crossbow.loading.middle @s")
					p.addTag("cooldown")
					system.runTimeout(()=>{
						p.removeTag("cooldown")
					}, 20)
				}
			}
		}
	}
}



export function item_animations(p) {
	let equip = p.getComponent(EntityEquippableComponent.componentId);
	try{
		let item = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(item){
			if(item.hasTag("template:shield") && !p.hasTag("mainhand") && !p.hasTag("offhand") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_main_hand root 100`)
				p.addTag("mainhand")
			}
			if(!item.hasTag("template:shield") || p.hasTag("offhand") || !p.isSneaking){
				if(p.hasTag("mainhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
					p.removeTag("mainhand")
				}
			}
			if(item.typeId.includes("new:") && item.typeId.includes("_trident") && p.hasTag("use_trident")){
				p.runCommandAsync(`playanimation @s animation.player.gem_trident root 100`)
				p.addTag("trident")
			}
			if(!item.typeId.includes("new:") || !item.typeId.includes("_trident") || !p.hasTag("use_trident")){
				if(p.hasTag("trident")){
					p.runCommandAsync("playanimation @s animation.player.gem_trident root")
					p.removeTag("trident")
				}
			}
			if(item.typeId.includes("new:") && item.typeId.includes("_bow") && p.hasTag("use_bow") && !p.hasTag("bow")){
				p.runCommandAsync(`playanimation @s animation.humanoid.custom_bow_and_arrow root 100000000`)
				p.addTag("bow")
			}
			if(!item.typeId.includes("new:") || !item.typeId.includes("_bow") || !p.hasTag("use_bow")){
				if(p.hasTag("bow")){
					p.runCommandAsync("playanimation @s animation.humanoid.custom_bow_and_arrow root")
					p.removeTag("bow")
				}
			}
		}
		if(!item){
			if(p.hasTag("mainhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
				p.removeTag("mainhand")
			}
			if(p.hasTag("trident")){
				p.runCommandAsync("playanimation @s animation.player.gem_trident root")
				p.removeTag("trident")
			}
			if(p.hasTag("bow")){
				p.runCommandAsync("playanimation @s animation.humanoid.custom_bow_and_arrow root")
				p.removeTag("bow")
			}
		}
	}catch(e){
		
	}
	try{
		let off = equip?.getEquipment(EquipmentSlot.Offhand);
		if(off){
			if(off.hasTag("template:shield")&&p.isSneaking){
				p.addTag("offhand")
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_off_hand root 100`)
			}
			if(!off.hasTag("template:shield")||!p.isSneaking){
				if(p.hasTag("offhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
					p.removeTag("offhand")
				}
			}
		}
		if(!off){
			if(p.hasTag("offhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
				p.removeTag("offhand")
			}
		}
	}catch(e){
		
	}
}
